'''
Created on Oct 27, 2023

@author: demunckd
'''

from enum import Enum


class Platform(Enum):
    PROBAV    = 'probav'
    SENTINEL3 = 'sen3'
    METEOT    = 'meteot'
class DB_platform(Enum):
    PROBAV      = 1
    S3A         = 2
    S3B         = 3
    S3          = 4    #S3 A+B
    METEOT      = 5

class DB_sensor(Enum):
    OLCI               = 1
    SLSTR              = 2
    OLCI_SLSTR         = 3
    METEOT             = 4
class SensorAbbr(Enum):
    PROBAV = 'PBV'
    SEN3   = 'S3'
    S3A    = 'S3A'
    S3B    = 'S3B'
    OLCI   = 'OLCI'
    METEOT = 'METEOT'
    
    def to_platform(self):
        if self == SensorAbbr.PROBAV:
            return Platform.PROBAV
        elif self == SensorAbbr.METEOT:
            return Platform.METEOT
        return Platform.SENTINEL3
    def to_db_platform(self):
        if self == SensorAbbr.PROBAV:
            return DB_platform.PROBAV
        elif self == SensorAbbr.METEOT:
            return DB_platform.METEOT
        elif self == SensorAbbr.S3A:
            return DB_platform.S3A
        elif self == SensorAbbr.S3B:
            return DB_platform.S3B
        elif self == SensorAbbr.SEN3:
            return DB_platform.S3
        elif self == SensorAbbr.OLCI:
            raise ValueError("OLCI is not a platform")
        else:
            raise ValueError("Unknown sensor abbreviation")


#no _ or - here please - to allow for parsing
class ProductType(Enum):
    TOA         = 'TOA'
    IDEPIX      = 'IDEPIX'
    CBDK        = 'CBDK'
    TOCR        = 'TOCR'
    GEO         = 'GEO'
    TOC         = 'TOC'
    BA300       = 'BA300'
    NDVI300     = 'NDVI300'
    LAI300      = 'LAI300'
    FCOVER300   = 'FCOVER300'
    FAPAR300    = 'FAPAR300'
    DMP300      = 'DMP300'
    TMAX1       = 'TMAX1'
    TMAX10      = 'TMAX10'
    PREVEBF     = 'PREVEBF'

class Subtype(Enum):
    """
    FRAME = 'FRAME'
    S1 = 'S1'
    S10 = 'S10'
    M1 = 'M1'
    M10 = 'M10'
    GFC = 'GFC'
    """
    QL = 'QL'
    K = 'K'

class Timeliness(Enum):
    NRT = 'NRT'
    NTC = 'NTC'

class Consolidation(Enum):
    RT0 = 'RT0'
    RT1 = 'RT1'
    RT2 = 'RT2'
    RT6 = 'RT6'

class Periodicity(Enum):
    FRAME = 0
    DAILY = 1
    FIVE = 5
    WEEKLY = 7
    DEKAD = 10
    MONTHLY = 30
    YEARLY = 365

class AncillaryDataInputCategory(Enum):
    CLIMATE =  'climato'
    AF      = 'activefire'
    DEM     = 'dem'
    LAND    = 'landmask'
    LUE     = 'land.use.efficiency'
    WB      = 'waterbodies'
    WFS     = 'workflow.specific'
    MISSING = 'missing.tiles'

class AncillaryDataSourceName(Enum):
    CAMS_NRT_SURF   = 'cams.nrt.surface.ext'
    METEOC          = 'meteoconsult'                        #climato
    AEROSOL         = 'aer.extinction.monthly'              #climato/merra
    PRESSURE        = 'surf.press.water.vapor.monthly'   #climato/merra
    VIIRS_NRT       = 'nrt'                                 #AF/VIIRS  shp_nrt  txt/nrt
    VIIRS_NTC       = 'ntc'                                 #AF/VIIRS  shp_ntc  txt/archive
    USGS            = 'usgs'                                 #dem
    CCI             = 'cci'                                 # LAND = same as landcover/cci/tiles and WB
    OLCI            = 'esa.olci.sc.adf'                     # LAND
    GAUL            = 'gaul'                                # LAND still used? previously used in BAV1 / GEO, now esa_olci landmask is used
    JRC             = 'jrc'                                 # LAND still used? BAV1
    PRIOR           = 'brdf.pror'                           # WFS NDVI300
    EBFNOEBF        = 'ebf.noebf'                           # WFS/s3_geo/ebf_noebf for NRT  #WFS/reproc/s3_geo/ebf_noebf for reprocessing
    PREVEBF         = 'prevebf'                             # WFS/s3_geo/prevebf for NRT    #WFS/reproc/s3_geo/prevebf for reprocessing
    NDVI300         = 's3.ndvi3'                            # MISSING
    GEO300          = 's3.geo'                              # MISSING
    BA300           = 's3.bav3'                             # MISSING

class FileTypeExtension(Enum):
    NC = 'nc'
    TIFF = 'tiff'
    PNG = 'png'     # wordt gemaakt tijdens globale mosaic. Kleiner dan tiff -  doel: gebruik in dashboard
    H5 = 'h5'       # momenteel nog veel intermediate data in hdf5 formaat ...
    NPY = 'npy'     # subtiles GEO kan mss aangepast worden naar npy.npz
    NPZ = 'npy.npz'     #subtiles GEO - compressed numpy
    JSON = 'json'   # brdf intermediate
    TXT = 'txt'
    SHP = 'shp'     # active fire
    VRT = 'vrt'     # brdf output
    CSV = 'csv'     # meteoconsult input data

class TocBandGroup(Enum):
    Oatoc = 'Oa_toc'
    Oa_toc_err = 'Oa_toc_err'
    S_an_toc = 'S_an_toc'
    S_an_toc_err = 'S_an_toc_err' 
    angles = 'angles'
    cloud_an = 'cloud_an'
    quality_flags = 'quality_flags'
    pixel_classif_flags = 'pixel_classif_flags'
    AC_process_flag = 'AC_process_flag'
    '''
    2024-08-11 13:31:08       5775 cgl_TOC_20220419234337_X35Y03_S3B_AC_process_flag_v2.3.4.tiff
2024-08-11 13:31:07    1948649 cgl_TOC_20220419234337_X35Y03_S3B_Oa_toc_err_v2.3.4.tiff
2024-08-11 13:31:07    3468369 cgl_TOC_20220419234337_X35Y03_S3B_Oa_toc_v2.3.4.tiff
2024-08-11 13:31:07     885083 cgl_TOC_20220419234337_X35Y03_S3B_S_an_toc_err_v2.3.4.tiff
2024-08-11 13:31:07    1271446 cgl_TOC_20220419234337_X35Y03_S3B_S_an_toc_v2.3.4.tiff
2024-08-11 13:31:07    1942661 cgl_TOC_20220419234337_X35Y03_S3B_angles_v2.3.4.tiff
2024-08-11 13:31:07     157964 cgl_TOC_20220419234337_X35Y03_S3B_cloud_an_v2.3.4.tiff
2024-08-11 13:31:08      13191 cgl_TOC_20220419234337_X35Y03_S3B_pixel_classif_flags_v2.3.4.tiff
2024-08-11 13:31:08      38916 cgl_TOC_20220419234337_X35Y03_S3B_quality_flags_v2.3.4.tiff
'''

class DB_status(Enum):
    FAILED = -1
    EXPECTED = 0
    AVAILABLE = 1
    ONHOLD = 2
    PROCESSING = 8
    PROCESSED = 9
    OBSOLETE = 10
    EMPTY = 99

class DB_product_type(Enum):
    OLCI_L1B    = 1
    SLSTR_L1B   = 2
    OLCI_IDEPIX = 3
    TOA_FRAME   = 4
    TOA_TILE    = 5
    TOC         = 6
    CAMS_NRT    = 7
    CAMS_NTC    = 8
    TOCR        = 9
    NDVI        = 10
    GEO_DAY     = 11
    LAI300      = 12
    FAPAR300    = 13
    FCOVER300   = 14
    GEO_PREVEBF = 15
    VIIRS_NRT   = 16
    VIIRS_NTC   = 17
    BA300_NRT   = 18
    BA300_NTC   = 19
    METEOCONSULT= 20
    DMP_MAX1    = 21
    DMP_MAX10   = 22
    DMP300      = 23
    TEST_PRODUCT = 9999
