from datetime import datetime
from cgls_cpe.common.helper import pad_two_digits
from cgls_cpe.common.helper import pad_nb_digits
from cgls_cpe.cpl.date_time import get_month_given_day_of_year

class Timestamp(object):

    def __init__(self, year:int, month:int, day:int, hour:int=-1, minutes:int=-1, seconds:int=-1):
        self.year = int(year)
        self.month = int(month)
        self.day = int(day)
        self.hour = int(hour)
        self.minutes = int(minutes)
        self.seconds = int(seconds)
        
    # Define getter methods for each attribute
    def get_year(self) -> int:
        return self.year

    def get_month(self) -> int:
        return self.month

    def get_day(self) -> int:
        return self.day

    def get_hour(self) -> int:
        return self.hour

    def get_minutes(self) -> int:
        return self.minutes

    def get_seconds(self) -> int:
        return self.seconds
    
    def __str__(self):
        return self.get_value()
    
    def get_date_short(self):
        return  ''.join([str(self.year) , pad_two_digits(self.month) , pad_two_digits(self.day)]) 
    
    def get_value(self):
        result = self.get_date_short()
        if self.hour >= 0:
            result += pad_two_digits(self.hour)
        if self.minutes>= 0:
            result += pad_two_digits(self.minutes)
        if self.seconds >= 0:
            result += pad_two_digits(self.seconds)
        return result
    
    def to_datetime(self):
        return datetime(self.year, self.month, self.day, self.hour, self.minutes, self.seconds)
    
    
class TimestampYearDay(Timestamp):
    def __init__(self, year:int, day_of_year:int):
        self.year = year
        self.day_of_year = day_of_year
        
    
    def get_month(self):
        return get_month_given_day_of_year(self.year,self.day_of_year)
    
    def get_date_short(self):
        return  self.get_value() 
    
    def get_value(self):
        result =  pad_nb_digits(self.year,4) + '_' + pad_nb_digits(self.day_of_year,3) 
        return result

    def to_datetime(self):
        return datetime.strptime(f'{self.year}{self.day_of_year:03}', '%Y%j')
    