import re
from cgls_cpe.exceptions.exception import InvalidParameterError


class Version(object):

    def __init__(self, major,minor,update, prefix=None):
        self.major = major
        self.minor = minor
        self.update = update
        self.prefix =''
        if prefix is not None and prefix != '':
            extracted_prefix,_ = Version.extract_prefix_and_update(prefix + str(update))
            if ( extracted_prefix != prefix):
                raise InvalidParameterError("Prefix should only be a-zA-Z")
            self.prefix= prefix
    
    def __str__(self):
        return self.get_value()
    
    def get_value(self):
        return 'v' + str(self.major) + '.' + str(self.minor) + '.' + self.prefix + str(self.update)       
    

    @staticmethod
    def extract_prefix_and_update(update_str):
        p = re.compile('([a-zA-Z]+)([0-9]+)')
        result = p.match(update_str)
        if result is None:
            return (None,update_str)
        return (result.group(1),result.group(2))

    @classmethod
    def from_string(cls, version_str):
        if version_str[0] == 'v':
            version_str = version_str[1:]
        
        splitted = version_str.split('.')
        major = splitted[0]       
        minor = splitted[1]
        prefix,update = Version.extract_prefix_and_update(splitted[2])
        return cls(major, minor, update, prefix)  