from cgls_cpe.storage import local_storage
from cgls_cpe.storage import remote_storage
from cgls_cpe.config.configuration import Configuration
from cgls_cpe.exceptions.exception import InvalidParameterError

import os
import glob
import json

from pyspark import SparkContext

from time import time, gmtime, strftime
from cgls_cpe.logging import log
from cgls_cpe.common.cdse_post_common import syncResultsToRemote, setupApplication, parseArgs,\
                                                    getBasePythonPath, createPostCfgDictList
import sys

TEMP_STRING='TEMPSTRING'

def run_job(cfgDict):
    logger = log.logger()
    
    try:#needed because we are in executor
        remote_storage.export_credentials_to_env()
        if sys.path[0] != getBasePythonPath() :
            sys.path.insert(0,getBasePythonPath())
        from cgls_cpe.cpl import mosaicNetcdf
        startTime = time()
        tmp_folder = local_storage.get_tempfolder_for_this_execute_run()
        
        tmp_input = os.path.join(tmp_folder, 'input')
        os.makedirs(tmp_input, exist_ok=True)
        remoteFileLst = cfgDict['input_files']
        remote_storage.download_list_of_keys_to_local_folder(remoteFileLst, tmp_input, overwrite=False, max_workers=5)
        logger.info("Done syncing to local folder" )
        fileLst = glob.glob(tmp_input+('/*'))
        logger.info(f" > Discovered {len(fileLst)} inputs in local storage")
        if len(fileLst) == 0:
            logger.error("Did not found any input products, leaving it here")
            raise InvalidParameterError("No products found! Check your config")
        cfgDict['input_files'] = fileLst
        
        tmp_output = os.path.join(tmp_folder, 'output')
        os.makedirs(tmp_output, exist_ok=True)
        cfgDict['out_folder_remote'] = os.path.dirname(cfgDict['output_file'])
        cfgDict['out_filename'] = os.path.basename(cfgDict['output_file'])
        cfgDict['tmpFolder'] = tmp_folder
        cfgDict['output_file'] =  os.path.join(tmp_output , cfgDict['out_filename'])
        mosaicNetcdf.createMosaicNetcdf(cfgDict, logger.debug)
        endTime = time()
        logger.info(f'... Ended processing in {strftime("%H:%M:%S", gmtime(endTime - startTime))}')
        logger.info("Now uploading the results..." )
        if os.path.exists(cfgDict['output_file']):
            syncResultsToRemote(cfgDict,logger.info)
            logger.info("Done uploading the results.")
        else:
            logger.error(f"Output file {cfgDict['output_file']} does not exist.")
    
    finally:
        local_storage.cleanup_tempfolder_for_this_execute()


if __name__ == '__main__':
    
    args = parseArgs(os.path.basename(__file__), 'Post processing (mosaic)')
    
    config = Configuration(args.cfgFile)
    config.dump_data_to_log()
    remote_storage.export_credentials_to_env()
    logger = log.logger()
    
    cfgDict = setupApplication(args,skip_output_replace=True)
    cfgDict["outputRoiFile"] = os.path.dirname(__file__) + '/../common/cgl_s3_roi-geo.json'
    cfgDict["inputRoiFile"] = os.path.dirname(__file__) + '/../common/cgl_s3_grid-geo.json'
    #cfgDict['settings']['tmpFolder'] = TEMP_STRING
     
    from cgls_cpe.common.cdse_post_common import createPostCfgDictList
    mosaicCfgList = createPostCfgDictList(cfgDict, logger.debug)
    nrMosaicToProcess = len(mosaicCfgList)
    startTime = time()
    if args.local:
        logger.info('Processing sequential')
        for index, mosaicCfgDict in enumerate(mosaicCfgList):
            logger.info(f' > {index + 1}/{nrMosaicToProcess}: {os.path.basename(mosaicCfgDict["output_file"])}')
            '''
            fn='/data/users/Private/gotenr/postcfg'+str(index)+'.json'
            with open(fn, 'w') as fn:
                json.dump(mosaicCfgDict, fn)
            '''
            run_job(mosaicCfgDict)
    else:
        logger.info('Setup Spark Context (pl_s3_post_cgl)')
        sc = SparkContext(appName=f"pl_s3_post_cgl")
        sc.setLogLevel("WARN")
        logger.info(f' > Application ID: {sc.applicationId}')
        try:
            sc.parallelize(mosaicCfgList, len(mosaicCfgList)).foreach(run_job)
            processTime = time() - startTime
        finally:     
            sc.stop()
    
    endTime = time()
    logger.info(f'... Ended processing in {strftime("%H:%M:%S", gmtime(endTime - startTime))}')
