'''
Created on Apr 24, 2024

@author: demunckd
'''
import time
from datetime import datetime
import argparse
from cgls_cpe.common.launcher_sandbox import load_sandbox_if_requested
from cgls_cpe.db_api import DatabaseConnection
from cgls_cpe.config.model.enums import DB_product_type, DB_status
from cgls_cpe.cpl import date_time


config = None
logger = None

def main():
    global config,logger
    #needed when we use an executor
    load_sandbox_if_requested()
    from cgls_cpe.logging import log
    from cgls_cpe.config.configuration import Configuration
    
    config = Configuration()
    logger =log.logger()
    
    parser = argparse.ArgumentParser(prog='wait_for_all_tocs_of_day')
    ### required arguments ###
    parser.add_argument('-s', '--startdate', type=str, 
                            required=True, 
                            help="start date in format 'YYYYMMDD'",
                            )
    parser.add_argument('-e', '--enddate', type=str, 
                            required=False, 
                            help="end date in format 'YYYYMMDD'",
                            )
    args =  parser.parse_args()
    enddate = args.startdate
    if args.enddate:
        enddate = args.enddate
    dateLst = date_time.listPeriod(args.startdate, enddate, period=1, first_day_only=True)
    
    for day_to_check in dateLst:    
        while True:
            logger.info("Going to check: %s", day_to_check)
            should_still_wait = wait_for_all_tocs(day_to_check)
            if should_still_wait:
                logger.info('Sleeping for 2 minutes...')
                time.sleep(120)
            else:
                logger.info("The wait is over")
                break
        
        

# returns True if we still need to wait        

def wait_for_all_tocs(date_to_check):
    
    config.dump_data_to_log()
    db_settings = config.get_database()
    DB          = DatabaseConnection(db_settings.get_connection_string())
    
    logger.info('Processing l2atotoc configure for date {}'.format(date_to_check))
    product_type = DB_product_type.OLCI_L1B.value
    status_id    = [DB_status.AVAILABLE.value]
    db_startdate = datetime(int(date_to_check[:4]), int(date_to_check[4:6]), int(date_to_check[6:8]))
    db_enddate   = datetime(int(date_to_check[:4]), int(date_to_check[4:6]), int(date_to_check[6:8]), 23, 59, 59)
    olci_list     = DB.get_products(product_type, status_id, startdate=db_startdate, enddate=db_enddate)
    logger.info("Found {} OLCI product(s) in DB for date {} with type {} and status {}.".format(len(olci_list), date_to_check, product_type, status_id))
    if len(olci_list) < config.get_input().get_minimal_number_of_olcis():
        logger.info("Only got %s olcis where minimum is %s so waiting...", len(olci_list), config.get_input().get_minimal_number_of_olcis())
        return True
    
    next_date = date_time.dateNextPeriod(date_to_check,1,outFormat= '%Y-%m-%d')
    query_base = "select count(*) from products where typeid = " + str(DB_product_type.OLCI_L1B.value) + " and periodstartdate > '" + next_date + "'"
    
    #TODO: if platform is registered, can be made more stable
    var_part = " and left(name, 3) = 'S3A'"
    logger.info("doing query to find next day: %s" , query_base + var_part)
    result = DB.doQuery(query_base + var_part, close_connection=False)
    count_s3a = result[0][0]
    
    var_part = " and left(name, 3) = 'S3B'"
    logger.info("doing query to find next day: %s" , query_base + var_part)
    result = DB.doQuery(query_base + var_part, close_connection=False)
    count_s3b = result[0][0]
    if ( count_s3a == 0 or count_s3b == 0):
        logger.info("Either S3A or S3B did not yet get to next day: %s" , next_date )
        return True
    else: 
        logger.info("Already %s S3A and %s S3B products for following day, so continue", count_s3a, count_s3b)
        
    product_type = DB_product_type.TOA_FRAME.value
    status_id    = [DB_status.PROCESSED.value]
    toa_list     = DB.get_products(product_type, status_id, startdate=db_startdate, enddate=db_enddate)
    logger.info("Found {} TOA frame(s) in DB for date {} with type {} and status {}.".format(len(toa_list), date_to_check, product_type, status_id))
    if len(toa_list) < len(olci_list) :
        logger.info("Only got %s TOAs where we got %s OLCIS so waiting...", len(toa_list), len(olci_list))
        return True
    
    
    product_type = DB_product_type.TOA_TILE.value
    toa_tile_list     = DB.get_products(product_type, status_id, startdate=db_startdate, enddate=db_enddate)
    logger.info("Found {} TOA tile(s) in DB for date {} with type {} and status {}.".format(len(toa_tile_list), date_to_check, product_type, status_id))
    
    product_type = DB_product_type.TOC.value
    toc_tile_list     = DB.get_products(product_type, status_id, startdate=db_startdate, enddate=db_enddate)
    logger.info("Found {} TOC product(s) in DB for date {} with type {} and status {}.".format(len(toc_tile_list), date_to_check, product_type, status_id))
    if len(toc_tile_list) < len(toa_tile_list) :
        logger.info("Only got %s TOCs where we got %s TOA tiles so waiting...", len(toc_tile_list), len(toa_tile_list))
        return True
    
    logger.info("all tests passed, stop the wait")
    return False

if __name__ == '__main__':
    main()
    
