'''
Created on Dec 8, 2023

@author: oomsb
'''
import datetime
import traceback
import os
import requests
import re
import time
import random

from cgls_cpe.config.configuration import Configuration
from cgls_cpe.common.helper import run_command

from cgls_cpe.logging import log
from cgls_cpe.storage.implementation.s3_api import S3Api
logger = log.logger()

KEY_ALTERNATIVE_LOC='alternative-s3-loc'

def check_environment(context):
    if context.is_production() != Configuration().is_production():
        raise Exception("Mixing up environments")

class PRIP():
    def __init__(self):
        # get configuration parameters
        self.config = Configuration()
        self.username = self.config.get_secrets().get_secret_key('PRIP_usr')
        self.password = self.config.get_secrets().get_secret_key('PRIP_pwd')
        
        #for temp storage
        store = Configuration().get_storage()
        self.tmp_s3 = S3Api(store.get_object_storage_endpoint(), self.config.get_secrets().get_secret_key('TEMP_S3_OL_KEY_ID'),
                           self.config.get_secrets().get_secret_key('TEMP_S3_OL_SECRET_KEY'))

    def do_request(self, url):
        r = requests.get(url, auth=(self.username, self.password), verify=False)
        return r

    def download_and_unzip(self, url, extract_to='.'):
        logger.debug("url: %s, extract to : %s" % (url, extract_to))
        alternative_failed = False
        if KEY_ALTERNATIVE_LOC in url:
            try:
                bucket_key_path= url[ url.index(KEY_ALTERNATIVE_LOC)+1+len(KEY_ALTERNATIVE_LOC):]
                logger.info(f"Using alternative temp storage s3://{bucket_key_path}")
                bucket = bucket_key_path[0:bucket_key_path.index('/')]
                key=bucket_key_path[bucket_key_path.index('/')+1:]
                local_filename= os.path.basename(key)
                self.tmp_s3.download_single_file(bucket, key, local_filename)
            except:
                logger.exception("Alternative S3 download failed, using alternative")
                alternative_failed = True
        
        if alternative_failed or KEY_ALTERNATIVE_LOC not in url:
            match = re.search(r'Products\(([^)]+)\)', url)
            id = match.group(1)
            local_filename = '%s.zip' % id
            max_retries = 10
            delay=60 #seconds to retry download * random number
    
            attempt = 0
            while attempt < max_retries:
                try:
                    with requests.get(url, auth=(self.username, self.password), verify=False, stream=True) as r:
                        r.raise_for_status()
                        with open(local_filename, 'wb') as f:
                            for chunk in r.iter_content(chunk_size=8192):
                                f.write(chunk)
                    logger.info(f"Downloaded {local_filename}")
                    break
                except Exception as e:
                    attempt += 1
                    logger.warning(f"Attempt {attempt} failed: Unable to download {url} to {local_filename}")
                    logger.warning(f"Response status code: {r.status_code if 'r' in locals() else 'N/A'}")
                    logger.warning(f"Error: {str(e)}")
                    
                    if attempt < max_retries:
                        time_to_sleep = random.random()*delay
                        logger.info(f"Retrying in {time_to_sleep} seconds...")
                        time.sleep(time_to_sleep)
                    else:
                        logger.exception("Giving up after exception")
                        logger.error("All attempts failed.")
                        return -1, ''

        
        logger.debug('local filename: %s '% local_filename)


        #get object name
        match = re.search("(S3.*\.SEN3).*", local_filename)
        if match is None:
            cmd = 'unzip -Z -1 %s | head -1' % local_filename
            stdout, stderr, err_nr = run_command(cmd, end_on_failure=True)
            logger.debug("stdout : %s" % stdout)
            match = re.search("b'(S3.*\.SEN3)", stdout)
        
        object_name = match.group(1)


        # Unzip the file using the system unzip command
        cmd = 'unzip -o %s -d %s' % (local_filename, extract_to)
        stdout, stderr, err_nr = run_command(cmd, end_on_failure=True)
        logger.debug(f"Extracted files to {extract_to}")

        # Optionally delete the zip file after extraction
        if os.path.exists(local_filename):
            os.remove(local_filename)
            logger.debug(f"Deleted {local_filename}")

        return 0, os.path.join(extract_to, object_name)

    def download(self, remote_path, local_path, exclude_lst=[]):
        logger.debug("Downloading %s" % remote_path)
        try:
            self.eo_data_s3.download_dir_from_s3('EODATA', remote_path, local_path, exclude_lst=exclude_lst)
            return 0
        except:
            logger.error("Unable to download %s to %s" % (remote_path, local_path))
            traceback.print_exc()
            return -1